   1  --
   2  -- Crawler
   3  -- This is the specialization for Crawlers
   4  --
   5  -- @author  Stefan Geiger
   6  -- @date  30/11/08
   7  --
   8  -- Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.
   9  
  10  Crawler = {};
  11  
  12  function Crawler.prerequisitesPresent(specializations)
  13      return SpecializationUtil.hasSpecialization(Steerable, specializations);
  14  end;
  15  
  16  function Crawler:load(xmlFile)
  17  
  18      Crawler.loadCrawlers(self, xmlFile);
  19  
  20      --[[local crawlerSound = getXMLString(xmlFile, "vehicle.crawlerSound#file");
  21      if crawlerSound ~= nil and crawlerSound ~= "" then
  22          crawlerSound = Utils.getFilename(crawlerSound, self.baseDirectory);
  23          self.crawlerSound = createSample("crawlerSound");
  24          self.crawlerSoundEnabled = false;
  25          loadSample(self.crawlerSound, crawlerSound, false);
  26          self.crawlerSoundPitch = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.crawlerSound#pitchOffset"), 1);
  27          self.crawlerSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.crawlerSound#volume"), 1);
  28      else
  29          self.crawlerSoundPitch = 1;
  30          self.crawlerSoundVolume = 1;
  31      end;]]
  32  
  33  end;
  34  
  35  function Crawler:loadCrawlers(xmlFile)
  36      self.crawlers = {};
  37      local i = 0;
  38      while true do
  39          local key = string.format("vehicle.crawlers.crawler(%d)", i);
  40          if not hasXMLProperty(xmlFile, key) then
  41              break;
  42          end;
  43          local node = Utils.indexToObject(self.components, getXMLString(xmlFile, key.."#index"));
  44          local speedRefNode = Utils.getNoNil(Utils.indexToObject(self.components, getXMLString(xmlFile, key.."#speedRefNode")), node);
  45          local scrollSpeed = Utils.getNoNil(getXMLFloat(xmlFile, key.."#scrollSpeed"), 1);
  46          local shaderParameterName = Utils.getNoNil(getXMLString(xmlFile, key.."#shaderParameterName"), "scrollPosition");
  47          local shaderParameterComponent = Utils.getNoNil(getXMLInt(xmlFile, key.."#shaderParameterComponent"), 1);
  48          local scrollLength = Utils.getNoNil(getXMLFloat(xmlFile, key.."#scrollLength"), 1);
  49          if node ~= nil then
  50              local rotatingParts = {};
  51              local j = 0;
  52              while true do
  53                  local keyRotation = key..string.format(".rotatingPart(%d)", j);
  54                  if not hasXMLProperty(xmlFile, keyRotation) then
  55                      break;
  56                  end;
  57                  local rotationNode = Utils.indexToObject(self.components, getXMLString(xmlFile, keyRotation.."#index"));
  58                  local speedScale = getXMLFloat(xmlFile, keyRotation.."#speedScale");
  59                  if speedScale == nil then
  60                      speedScale = 1.0/Utils.getNoNil(getXMLFloat(xmlFile, keyRotation.."#radius"), 1);
  61                  end;
  62                  if rotationNode ~= nil then
  63                      table.insert(rotatingParts, {node=rotationNode, speedScale=speedScale});
  64                  end;
  65                  j = j + 1;
  66              end;
  67  
  68              table.insert(self.crawlers, {node=node, scrollSpeed=scrollSpeed, scrollPosition=0, scrollLength=scrollLength, shaderParameterName=shaderParameterName, shaderParameterComponent=shaderParameterComponent, speedRefNode=speedRefNode, rotatingParts=rotatingParts});
  69          end;
  70          i = i + 1;
  71      end;
  72  end;
  73  
  74  function Crawler:delete()
  75  
  76      --[[if self.crawlerSound ~= nil then
  77          delete(self.crawlerSound);
  78      end;]]
  79  
  80  end;
  81  
  82  function Crawler:mouseEvent(posX, posY, isDown, isUp, button)
  83  end;
  84  
  85  function Crawler:keyEvent(unicode, sym, modifier, isDown)
  86  
  87      --[[if isDown then
  88          if sym == Input.KEY_9 then
  89              self.uvScrollSpeed = self.uvScrollSpeed + 0.001;
  90              print(self.uvScrollSpeed);
  91          elseif sym == Input.KEY_8 then
  92              self.uvScrollSpeed = self.uvScrollSpeed - 0.001;
  93              print(self.uvScrollSpeed);
  94          end;
  95      end;]]
  96  
  97  end;
  98  
  99  function Crawler:update(dt)
 100  
 101  --    local currentSpeed = self.lastSpeed*3600;
 102  --    print ("currentSpeed " , self.offsetUVx );
 103  
 104      if self:getIsActive() then
 105  
 106  
 107          for _, crawler in pairs(self.crawlers) do
 108              local newX, newY, newZ = getWorldTranslation(crawler.speedRefNode);
 109              if crawler.lastPosition == nil then
 110                  crawler.lastPosition = {newX, newY, newZ};
 111              end;
 112              local dx, dy, dz = worldDirectionToLocal(crawler.speedRefNode, newX-crawler.lastPosition[1], newY-crawler.lastPosition[2], newZ-crawler.lastPosition[3]);
 113              local movingDirection = 0;
 114              if dz > 0.0001 then
 115                  movingDirection = 1;
 116              elseif dz < -0.0001 then
 117                  movingDirection = -1;
 118              end;
 119              local movedDistance = Utils.vector3Length(dx, dy, dz);
 120              crawler.lastPosition[1] = newX;
 121              crawler.lastPosition[2] = newY;
 122              crawler.lastPosition[3] = newZ;
 123  
 124              crawler.scrollPosition = (crawler.scrollPosition + movedDistance*movingDirection*crawler.scrollSpeed) % crawler.scrollLength;
 125              if crawler.shaderParameterComponent == 1 then
 126                  setShaderParameter(crawler.node, crawler.shaderParameterName, crawler.scrollPosition,0,0,0, false);
 127              else
 128                  setShaderParameter(crawler.node, crawler.shaderParameterName, 0,crawler.scrollPosition,0,0, false);
 129              end;
 130  
 131              for _, rotatingPart in pairs(crawler.rotatingParts) do
 132                  rotate(rotatingPart.node, rotatingPart.speedScale * movedDistance * movingDirection, 0, 0);
 133              end;
 134          end;
 135  
 136          --[[if not self.crawlerSoundEnabled then
 137              playSample(self.crawlerSound, 0, 0, 0);
 138              self.crawlerSoundEnabled = true;
 139          end;
 140  
 141          if self.crawlerSound ~= nil then
 142              local maxDistance  = 0.15;
 143              if self.crawlerSoundEnabled then
 144                  local crawlerVolume = Utils.clamp(( math.abs(movedDistance) / maxDistance), 0.0, 1.0);
 145                  setSampleVolume(self.crawlerSound, crawlerVolume-0.75);
 146              end;
 147          end;]]
 148  
 149      end;
 150  
 151  end;
 152  
 153  function Crawler:draw()
 154  end;
 155  
 156  function Crawler:developmentReloadFromXML(xmlFile)
 157      Crawler.loadCrawlers(self, xmlFile);
 158  end;
